// SerialPort.cpp: implementation of the SerialPort class.
// CUSTOMIZED FOR DEV-CPP; MIGHT HAVE TO BE MODIFIED FOR VISUAL C++ AGAIN
//////////////////////////////////////////////////////////////////////

#include "SerialPort.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SerialPort::SerialPort()
{
	ok=false;
	port="Com1";
	baud=9600;
	bytes=8;
	parity=NOPARITY;
	stopBits=ONESTOPBIT;
}

SerialPort::~SerialPort()
{

}

int SerialPort::open()
{
	ok=true;

	handle=CreateFile(port.c_str(),
		GENERIC_READ|GENERIC_WRITE,
		0,	//exclusive access
		NULL,	//no security
		OPEN_EXISTING,
		0,	//no overlapped I/O
		NULL);	//null template

	if(handle==INVALID_HANDLE_VALUE)
	{
		ok=false;
		return -1;
	}

	if(ok)
	{
		// set buffer sizes
		ok=SetupComm(handle, 128, 128);
		if(!ok)
			return -1;
	}

	if(ok)
	{
		// set up DCB
		ok=GetCommState(handle, &dcb);
		if(!ok)
			return -1;
	}

	if(ok)
	{
		dcb.BaudRate=baud;
		dcb.ByteSize=bytes;
		dcb.Parity=parity;
		dcb.StopBits=stopBits;
		dcb.fAbortOnError=TRUE;
		ok=SetCommState(handle, &dcb);
		if(!ok)
			return -1;
	}

	if(ok)
	{
		ok=GetCommTimeouts(handle, &timeouts);
		if(!ok)
			return -1;
	}

	if(ok)
	{
		timeouts.ReadIntervalTimeout=50;
		timeouts.ReadTotalTimeoutConstant=50;
		timeouts.ReadTotalTimeoutMultiplier=10;
		timeouts.WriteTotalTimeoutConstant=50;
		timeouts.WriteTotalTimeoutMultiplier=10;
		ok=SetCommTimeouts(handle, &timeouts);
		if(!ok)
			return -1;
	}

	return 0;
}

int SerialPort::close()
{
	CloseHandle(handle);
	return 0;
}

int SerialPort::readBytes(char *buf, int size)
{
	if(!ok)
		return -1;

	bool chk=false;
	unsigned long bytesRead=0;
	chk=ReadFile(handle, buf, size, &bytesRead, NULL);
	if(!chk || bytesRead<=0)
		return -1;

	return bytesRead;
}

int SerialPort::writeBytes(char *buf, int size)
{
	if(!ok)
		return -1;

	bool chk=false;
	unsigned long bytesWritten=0;
	chk=WriteFile(handle, buf, size, &bytesWritten, NULL);
	if(!chk || bytesWritten<=0)
		return -1;

	return bytesWritten;
}

int SerialPort::setPort(std::string port1)
{
	port=port1;
	return 0;
}
int SerialPort::setBaud(unsigned long baud1)
{
	baud=baud1;
	return 0;
}
int SerialPort::setBytes(unsigned char bytes1)
{
	bytes=bytes1;
	return 0;
}
int SerialPort::setStopBits(unsigned char stopBits1)
{
	stopBits=stopBits1;
	return 0;
}
int SerialPort::setParity(unsigned char parity1)
{
	parity=parity1;
	return 0;
}
